package sun.java2d.opengl;

import sun.java2d.pipe.Region;
import java.awt.Rectangle;
import java.awt.GraphicsConfiguration;
import sun.java2d.SurfaceData;
import sun.java2d.pipe.BufferedContext;
import sun.java2d.SunGraphics2D;
import java.awt.Graphics;

class OGLUtilities
{
    public static final int UNDEFINED = 0;
    public static final int WINDOW = 1;
    public static final int TEXTURE = 3;
    public static final int FLIP_BACKBUFFER = 4;
    public static final int FBOBJECT = 5;
    
    private OGLUtilities() {
    }
    
    public static boolean isQueueFlusherThread() {
        return OGLRenderQueue.isQueueFlusherThread();
    }
    
    public static boolean invokeWithOGLContextCurrent(Graphics graphics, Runnable runnable) {
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            if (graphics != null) {
                if (!(graphics instanceof SunGraphics2D)) {
                    return false;
                }
                SurfaceData surfaceData = ((SunGraphics2D)graphics).surfaceData;
                if (!(surfaceData instanceof OGLSurfaceData)) {
                    return false;
                }
                BufferedContext.validateContext((OGLSurfaceData)surfaceData);
            }
            instance.flushAndInvokeNow(runnable);
            OGLContext.invalidateCurrentContext();
        }
        finally {
            instance.unlock();
        }
        return true;
    }
    
    public static boolean invokeWithOGLSharedContextCurrent(GraphicsConfiguration graphicsConfiguration, Runnable runnable) {
        if (!(graphicsConfiguration instanceof OGLGraphicsConfig)) {
            return false;
        }
        OGLRenderQueue instance = OGLRenderQueue.getInstance();
        instance.lock();
        try {
            OGLContext.setScratchSurface((OGLGraphicsConfig)graphicsConfiguration);
            instance.flushAndInvokeNow(runnable);
            OGLContext.invalidateCurrentContext();
        }
        finally {
            instance.unlock();
        }
        return true;
    }
    
    public static Rectangle getOGLViewport(Graphics graphics, int width, int height) {
        if (!(graphics instanceof SunGraphics2D)) {
            return null;
        }
        SunGraphics2D sunGraphics2D = (SunGraphics2D)graphics;
        SurfaceData surfaceData = sunGraphics2D.surfaceData;
        int transX = sunGraphics2D.transX;
        int transY = sunGraphics2D.transY;
        Rectangle bounds = surfaceData.getBounds();
        return new Rectangle(transX, bounds.height - (transY + height), width, height);
    }
    
    public static Rectangle getOGLScissorBox(Graphics graphics) {
        if (!(graphics instanceof SunGraphics2D)) {
            return null;
        }
        SunGraphics2D sunGraphics2D = (SunGraphics2D)graphics;
        SurfaceData surfaceData = sunGraphics2D.surfaceData;
        Region compClip = sunGraphics2D.getCompClip();
        if (!compClip.isRectangular()) {
            return null;
        }
        int loX = compClip.getLoX();
        int loY = compClip.getLoY();
        int width = compClip.getWidth();
        int height = compClip.getHeight();
        Rectangle bounds = surfaceData.getBounds();
        return new Rectangle(loX, bounds.height - (loY + height), width, height);
    }
    
    public static Object getOGLSurfaceIdentifier(Graphics graphics) {
        if (!(graphics instanceof SunGraphics2D)) {
            return null;
        }
        return ((SunGraphics2D)graphics).surfaceData;
    }
    
    public static int getOGLSurfaceType(Graphics graphics) {
        if (!(graphics instanceof SunGraphics2D)) {
            return 0;
        }
        SurfaceData surfaceData = ((SunGraphics2D)graphics).surfaceData;
        if (!(surfaceData instanceof OGLSurfaceData)) {
            return 0;
        }
        return ((OGLSurfaceData)surfaceData).getType();
    }
    
    public static int getOGLTextureType(Graphics graphics) {
        if (!(graphics instanceof SunGraphics2D)) {
            return 0;
        }
        SurfaceData surfaceData = ((SunGraphics2D)graphics).surfaceData;
        if (!(surfaceData instanceof OGLSurfaceData)) {
            return 0;
        }
        return ((OGLSurfaceData)surfaceData).getTextureTarget();
    }
}
